/* Any copyright is dedicated to the Public Domain.
   http://creativecommons.org/publicdomain/zero/1.0/ */

/* import-globals-from head_search.js */

"use strict";

Services.prefs.setBoolPref("browser.search.modernConfig", false);
